import json
from typing import List

from eml_analyzer.library.outputs.abstract_output import AbstractOutput
from eml_analyzer.library.parser import ParsedEmail, Attachment, StructureItem


class JsonOutput(AbstractOutput):
    def __init__(self):
        self._result_dictionary = dict()

    def process_option_show_header(self, parsed_email: ParsedEmail) -> None:
        header_dictionary = dict()
        for key, value in parsed_email.get_header():
            if key in header_dictionary:
                header_dictionary[key].append(value)
            else:
                header_dictionary[key] = [value]
        self._result_dictionary["headers"] = header_dictionary

    def process_option_show_structure(self, parsed_email: ParsedEmail) -> None:
        structure_item = parsed_email.get_structure()
        self._result_dictionary["structure"] = JsonOutput._generate_dict_from_structure_item(structure_item=structure_item)

    @staticmethod
    def _generate_dict_from_structure_item(structure_item: StructureItem) -> dict:
        result_dict = {
            'type': structure_item.content_type,
        }

        if structure_item.filename is not None:
            result_dict['name'] = structure_item.filename
        if structure_item.content_disposition is not None:
            result_dict['disposition'] = structure_item.content_disposition

        if len(structure_item.child_items) > 0:
            child_dicts = list()
            for child in structure_item.child_items:
                child_dicts.append(JsonOutput._generate_dict_from_structure_item(structure_item=child))
            result_dict['children'] = child_dicts

        return result_dict

    def process_option_show_embedded_urls_in_html_and_text(self, parsed_email: ParsedEmail) -> None:
        self._result_dictionary["urls"] = parsed_email.get_embedded_clickable_urls_from_html_and_text()

    def process_option_show_html(self, parsed_email: ParsedEmail) -> None:
        html = parsed_email.get_html_content()
        if html is not None:
            self._result_dictionary["html"] = html

    def process_option_show_text(self, parsed_email: ParsedEmail) -> None:
        text = parsed_email.get_text_content()
        if text is not None:
            self._result_dictionary["text"] = text

    def process_option_show_attachments(self, parsed_email: ParsedEmail, extract_content: bool = False) -> None:
        attachment_list = parsed_email.get_attachments()
        self._result_dictionary["attachments"] = JsonOutput._generate_attachments_dict_from_attachment_list(attachment_list=attachment_list, extract_content=extract_content)

    @staticmethod
    def _generate_attachments_dict_from_attachment_list(attachment_list: List[Attachment], extract_content: bool) -> List[dict]:
        result_list = list()
        attachment: Attachment
        for attachment in attachment_list:
            result_list.append(JsonOutput._generate_attachment_dict_from_attachment(attachment=attachment, extract_content=extract_content))
        return result_list

    @staticmethod
    def _generate_attachment_dict_from_attachment(attachment: Attachment, extract_content: bool) -> dict:
        attachment_dict = {
            'type': attachment.content_type,
        }
        if attachment.filename is not None:
            attachment_dict['name'] = attachment.filename
        if attachment.content_disposition is not None:
            attachment_dict['disposition'] = attachment.content_disposition
        if extract_content:
            attachment_dict['content_in_base64'] = attachment.get_content_base64_encoded()
        return attachment_dict

    def process_option_show_reloaded_content_from_html(self, parsed_email: ParsedEmail) -> None:
        self._result_dictionary["reloaded_content"] = parsed_email.get_reloaded_content_from_html()

    def output_error(self, exception: Exception, error_message: str) -> None:
        error_dict = {
            'error_message': error_message,
        }
        if exception:
            error_dict['exception'] = str(exception)
        print(json.dumps(error_dict, indent=4))

    def get_final_output(self, parsed_email: ParsedEmail) -> str or None:
        error_messages: List[str] = parsed_email.get_error_messages()
        if len(error_messages) > 0:
            self._result_dictionary['warnings'] = parsed_email.get_error_messages()
        return json.dumps(self._result_dictionary, indent=4)



from logging import warning

from cli_formatter.output_formatting import print_headline_banner, colorize_string, Color, info, error, warning

from eml_analyzer.library.outputs.abstract_output import AbstractOutput
from eml_analyzer.library.parser import ParsedEmail, StructureItem, Attachment


class StandardOutput(AbstractOutput):
    def __int__(self):
        pass

    def get_final_output(self, parsed_email: ParsedEmail) -> str or None:
        error_messages = parsed_email.get_error_messages()
        if len(error_messages) > 0:
            print()
            for message in error_messages:
                warning(message=message)
            print()
        return

    def process_option_show_header(self, parsed_email: ParsedEmail) -> None:
        print_headline_banner(headline='Header')
        max_key_width = max([len(x) for x, _ in parsed_email.get_header()])
        for key, value in parsed_email.get_header():
            values_in_lines = value.split('\n')
            first_value = values_in_lines.pop(0)
            print(colorize_string(text=key, color=Color.CYAN) + (max_key_width - len(key) + 5) * '.' + first_value)
            for x in values_in_lines:
                x = x.replace('\t', '').strip().replace('\r', '').strip(' ')
                print((max_key_width + 5) * ' ' + x)
        print()

    def process_option_show_structure(self, parsed_email: ParsedEmail) -> None:
        print_headline_banner(headline='Structure')
        structure: StructureItem = parsed_email.get_structure()
        self._print_structure(structure=structure)
        print()

    def _print_structure(self, structure: StructureItem, level: int = 0):
        filename = ('  [' + colorize_string(text=structure.filename, color=Color.CYAN) + ']') if structure.filename is not None else ''
        type_with_intend = level * '|  ' + '|- {}'.format(structure.content_type)
        print(type_with_intend.ljust(40), filename)
        for child_item in structure.child_items:
            self._print_structure(structure=child_item, level=level + 1)

    def process_option_show_embedded_urls_in_html_and_text(self, parsed_email: ParsedEmail) -> None:
        print_headline_banner(headline='URLs in HTML and text part')
        all_links = parsed_email.get_embedded_clickable_urls_from_html_and_text()
        if len(all_links) == 0:
            info(message='No URLs were found')
        for x in all_links:
            print(' - ' + colorize_string(text=x, color=Color.MAGENTA))
        print()

    def process_option_show_html(self, parsed_email: ParsedEmail) -> None:
        print_headline_banner(headline='HTML')
        html = parsed_email.get_html_content()
        if html is None:
            info('Email contains no HTML')
        else:
            print(html)
        print()

    def process_option_show_text(self, parsed_email: ParsedEmail) -> None:
        print_headline_banner(headline='Plaintext')
        text = parsed_email.get_text_content()
        if text is None:
            info('Email contains no plaintext')
        else:
            print(text)
        print()

    def process_option_show_attachments(self, parsed_email: ParsedEmail, extract_content: bool = False) -> None:
        print_headline_banner('Attachments')
        attachments_table_rows = list()
        attachment: Attachment
        for attachment in parsed_email.get_attachments():
            attachments_table_rows.append((attachment.filename, attachment.content_type, attachment.content_disposition))
        if len(attachments_table_rows) == 0:
            info('E-Mail contains no attachments')
        else:
            max_width_filename = max([len(filename) for (filename, content_type, disposition) in attachments_table_rows]) + 7
            max_width_content_type = max([len(content_type) for (filename, content_type, disposition) in attachments_table_rows]) + 7
            for index, (filename, content_type, disposition) in enumerate(attachments_table_rows):
                index_str = '[' + colorize_string(text=str(index + 1), color=Color.CYAN) + ']'
                print(index_str, filename.ljust(max_width_filename), content_type.ljust(max_width_content_type), disposition)
        print()

    def process_option_show_reloaded_content_from_html(self, parsed_email: ParsedEmail) -> None:
        print_headline_banner(headline='Reloaded Content (aka. Tracking Pixels)')
        sources = parsed_email.get_reloaded_content_from_html()
        if parsed_email.get_html_content() is None:
            warning('Email contains no HTML')
        else:
            if len(sources) == 0:
                info(message='No content found which will be reloaded from external resources')
            for x in sources:
                print(' - ' + colorize_string(text=x, color=Color.MAGENTA))
        print()

    def output_error(self, exception: Exception or None, error_message: str) -> None:
        if exception:
            error('Error: {}'.format(exception))
        error(error_message)
        info('Exiting')



import re
import email
import email.message
import urllib.parse
import html
import warnings
from typing import NamedTuple, List, Tuple, Set

from eml_analyzer.library.parser.attachment import Attachment
from eml_analyzer.library.parser.printable_filename import decode_ASCII_encoded_string
from eml_analyzer.library.parser.structure_item import StructureItem


class EmlParsingException(Exception):
    pass


class PayloadDecodingException(Exception):
    pass


class FoundUrl(NamedTuple):
    url: str
    original: str


class ParsedEmail:
    def __init__(self, eml_content: str):
        self._parsed_email = ParsedEmail._parse_email(eml_content=eml_content)

        self._html_was_analyzed: bool = False
        self._html_content: str or None = None

        # an list containing error messages which occurred during parsing
        self._error_messages: List[str] = list()

    @staticmethod
    def _parse_email(eml_content: str) -> email.message.Message:
        try:
            return email.message_from_string(eml_content)
        except Exception as e:
            raise EmlParsingException(e)

    def get_error_messages(self) -> List[str]:
        return self._error_messages

    def _add_error_messages(self, error_message: str) -> None:
        self._error_messages.append(error_message)

    def get_header(self) -> List[Tuple[str, any]]:
        """ returns list of key-value pairs of header entries """
        return [(key, decode_ASCII_encoded_string(value)) for key, value in self._parsed_email.items()]

    def get_structure(self) -> StructureItem:
        return StructureItem(message=self._parsed_email)

    def get_text_content(self) -> str or None:
        return self._get_decoded_payload_with_first_matching_type(content_type='text/plain')

    def get_html_content(self) -> str or None:
        # check if the html was already extracted in the past to prevent double work
        if not self._html_was_analyzed:
            self._html_content = self._get_decoded_payload_with_first_matching_type(content_type='text/html')
        return self._html_content

    def _get_decoded_payload_with_first_matching_type(self, content_type: str) -> str or None:
        first_matched_payload = ParsedEmail._get_first_email_payload_with_matching_type(message=self._parsed_email, content_type=content_type)
        if first_matched_payload is not None:
            try:
                return ParsedEmail._get_decoded_payload_from_message(message=first_matched_payload)
            except PayloadDecodingException:
                self._add_error_messages(error_message='Payload with the type "{}" could not be decoded'.format(content_type))
        return None

    @staticmethod
    def _get_first_email_payload_with_matching_type(message: email.message.Message, content_type: str) -> email.message.Message or None:
        if message.get_content_type() == content_type:
            return message

        child_payloads = message.get_payload()
        if type(child_payloads) is list:
            for child_payload in child_payloads:
                found_payload = ParsedEmail._get_first_email_payload_with_matching_type(message=child_payload, content_type=content_type)
                if found_payload is not None:
                    return found_payload
        return None

    @staticmethod
    def _get_decoded_payload_from_message(message: email.message.Message) -> None or str:
        transfer_encoding = ParsedEmail._header_lookup_first_element(message=message, key='content-transfer-encoding')
        if transfer_encoding in {'7bit', '8bit', 'binary'}:
            return message.get_payload(decode=False)

        payload_in_bytes = message.get_payload(decode=True)

        list_of_possible_encodings = ParsedEmail._create_list_of_possible_encodings(message=message)

        for encoding_format in list_of_possible_encodings:
            try:
                return payload_in_bytes.decode(encoding_format)
            except ValueError:
                continue
        raise PayloadDecodingException('Payload could not be decoded')

    @staticmethod
    def _create_list_of_possible_encodings(message: email.message.Message) -> list:
        """ creates a list of the most possible encodings of a payload """
        list_of_possible_encodings = list()

        header_values = ParsedEmail._header_lookup(message=message, key='content-type')

        # at first add the encodings mentioned in the object header
        for v in header_values:
            entries = v.split(';')
            for entry in entries:
                entry = entry.strip()
                if entry.startswith('charset='):
                    encoding = entry.replace('charset=', '').replace('"', '')
                    list_of_possible_encodings.append(encoding)

        for x in ['utf-8', 'windows-1251', 'iso-8859-1', 'us-ascii', 'iso-8859-15']:
            if x not in list_of_possible_encodings:
                list_of_possible_encodings.append(x)
        return list_of_possible_encodings

    @staticmethod
    def _payload_needs_decoding(message: email.message.Message) -> bool:
        transfer_encoding = ParsedEmail._header_lookup_first_element(message=message, key='content-transfer-encoding')
        if transfer_encoding is None:
            return True
        return transfer_encoding not in {'7bit', '8bit', 'binary'}

    @staticmethod
    def _header_lookup_first_element(message: email.message.Message, key: str) -> str or None:
        for header_key, value in message.items():
            if str(header_key).lower() == key:
                return str(value).lower()
        return None

    @staticmethod
    def _header_lookup(message: email.message.Message, key: str) -> [str]:
        values = list()
        for header_key, value in message.items():
            if str(header_key).lower() == key:
                values.append(str(value).lower())
        return values

    def get_attachments(self) -> List[Attachment]:
        return_list = list()
        counter = 0
        for child in self._parsed_email.walk():
            if child.get_filename() is not None:
                counter += 1
                return_list.append(Attachment(message=child, index=counter))
        return return_list

    def get_embedded_urls_from_html_and_text(self) -> List[str]:
        warnings.warn(
            "get_embedded_urls_from_html_and_text is deprecated, use get_embedded_clickable_urls_from_html_and_text instead",
            DeprecationWarning
        )
        return self.get_embedded_clickable_urls_from_html_and_text()

    def get_embedded_clickable_urls_from_html_and_text(self) -> List[str]:
        found_urls = set()
        html_data: str or None = self.get_html_content()
        if html_data is not None:
            found_urls.update(ParsedEmail._get_embedded_clickable_urls_from_html(html_data=html_data))
        text: str or None = self.get_text_content()
        if text is not None:
            found_urls.update(ParsedEmail._get_embedded_urls_from_text(text=text))
        return list(found_urls)

    @staticmethod
    def _get_embedded_clickable_urls_from_html(html_data: str) -> Set[str]:
        # find urls which are referenced in html links
        found_urls_in_html_links = ParsedEmail._get_embedded_clickable_urls_from_html_links(html_data=html_data)
        found_urls_in_html_text = ParsedEmail._get_embedded_clickable_urls_from_html_text(html_data=html_data, found_urls=found_urls_in_html_links)

        found_urls = found_urls_in_html_text
        found_url: FoundUrl
        for found_url in found_urls_in_html_links:
            found_urls.add(found_url.url)
        return found_urls

    @staticmethod
    def _get_embedded_clickable_urls_from_html_links(html_data: str) -> [FoundUrl]:
        """ extracts URLs from HTML links """
        found_urls = list()
        for pattern_template in [r' href="(.+?)"', r" href='(.+?)'", r" originalsrc='(.+?)'", r' originalsrc="(.+?)"']:
            pattern = re.compile(pattern_template, re.IGNORECASE)
            for match in re.finditer(pattern, html_data):
                extracted_url = match.group(1)
                decoded_url = html.unescape(extracted_url)
                decoded_url = urllib.parse.unquote(decoded_url)
                found_url = FoundUrl(decoded_url, extracted_url)
                found_urls.append(found_url)
        return found_urls

    @staticmethod
    def _get_embedded_clickable_urls_from_html_text(html_data: str, found_urls: [FoundUrl]) -> Set[str]:
        """ extracts URLs from HTML to capture also URLs which are embedded into the text rendered by the HTML payload """
        # remove at first the reloaded content from the html to prevent that URL from reloaded content are added to the embedded clickable URLs
        html_data = ParsedEmail._get_new_html_without_reloaded_content(html_data=html_data)
        # then remove the already found URLs
        found_url: FoundUrl
        for found_url in found_urls:
            html_data = html_data.replace(found_url.original, '')

        return ParsedEmail._get_embedded_urls_from_text(text=html_data)

    @staticmethod
    def _get_embedded_urls_from_text(text: str) -> Set[str]:
        found_urls = set()
        pattern = re.compile(r'(http|https|ftp|ftps)\:\/\/[a-zA-Z0-9\-\.]+\.[a-zA-Z]{2,3}(\/\S*)?', re.IGNORECASE)
        for match in re.finditer(pattern, text):
            found_urls.add(match.group(0))
        return found_urls

    def get_reloaded_content_from_html(self) -> List[str]:
        html_data: str or None = self.get_html_content()
        if html_data is not None:
            return ParsedEmail._get_reloaded_content_from_html(html_data=html_data)
        return list()

    @staticmethod
    def _get_reloaded_content_from_html(html_data: str) -> List[str]:
        sources = list()
        for pattern_template in ParsedEmail._get_patterns_for_reloaded_content():
            pattern = re.compile(pattern_template, re.IGNORECASE)
            for match in re.finditer(pattern, html_data):
                extracted_url = match.group(1)
                # embedded items which are attached to the email as attachment are referred to with a staring 'cid:', so these will be ignored
                if not extracted_url.startswith('cid:') and not extracted_url.startswith('data:'):
                    # decode URL
                    decoded_url: str = urllib.parse.unquote(extracted_url)
                    sources.append(decoded_url)
        return sources

    @staticmethod
    def _get_new_html_without_reloaded_content(html_data: str) -> str:
        for pattern_template in ParsedEmail._get_patterns_for_reloaded_content():
            pattern = re.compile(pattern_template, re.IGNORECASE)
            for match in re.finditer(pattern, html_data):
                html_data = html_data.replace(match.group(0), '')
        return html_data

    @staticmethod
    def _get_patterns_for_reloaded_content() -> List[str]:
        return [r' src="(.+?)"', r" src='(.+?)'", r' background="(.+?)"', r" background='(.+?)'"]



import email.message
import base64

from eml_analyzer.library.parser.printable_filename import get_printable_filename_if_existent


class Attachment:
    def __init__(self, message: email.message.Message, index: int):
        self.index: int = index
        self.filename: str or None = get_printable_filename_if_existent(message=message)
        self.content_type: str = message.get_content_type()
        self.content_disposition: str = message.get_content_disposition()
        self.content: bytes = message.get_payload(decode=True)

    def get_content_base64_encoded(self) -> str:
        return base64.b64encode(self.content).decode()



import base64
import binascii
import re
import email.message
import quopri


def get_printable_filename_if_existent(message: email.message.Message) -> str or None:
    filename = message.get_filename()
    if filename is None:
        return None
    return _make_string_printable(original_string=filename)


def _make_string_printable(original_string: str) -> str:
    original_string = decode_ASCII_encoded_string(string=original_string)

    additional_allowed_chars = {'_', '.', '(', ')', '-', ' '}
    clean_name = ''
    for x in original_string:
        if x.isalpha() or x.isalnum() or x in additional_allowed_chars:
            clean_name += x
        elif x.isprintable():
            clean_name += '_'
    return clean_name


def decode_ASCII_encoded_string(string: str) -> str:
    string = _decode_ASCII_encoded_string_baseX(string=string)
    string = _decode_ASCII_encoded_string_quoted_printable_string(string=string)
    return string


def _decode_ASCII_encoded_string_baseX(string: str) -> str:
    """ decodes ASCII strings which are encoded like: name := "=?UTF-8?B?" + base64_encode(string) + "?=" """
    pattern = re.compile(r'=\?(.+?)\?B\?(.+?)\?=', re.IGNORECASE)
    for match in list(re.finditer(pattern=pattern, string=string)):
        try:
            string = string.replace(match.group(0), base64.b64decode(match.group(2)).decode(match.group(1)))
        except binascii.Error:
            pass
    return string


def _decode_ASCII_encoded_string_quoted_printable_string(string: str) -> str:
    pattern = re.compile(r'=\?(.+?)\?Q\?(.+?)\?=', re.IGNORECASE)
    for match in list(re.finditer(pattern=pattern, string=string)):
        try:
            encoding = match.group(1)
            encoded_string = match.group(2)
            decoded_string = quopri.decodestring(encoded_string)
            replacement = decoded_string.decode(encoding)
            string = string.replace(match.group(0), replacement)
        except binascii.Error:
            pass
    return string



import email
from eml_analyzer.library.parser.printable_filename import get_printable_filename_if_existent


class StructureItem:
    def __init__(self, message: email.message.Message):
        self.content_type = message.get_content_type()
        self.filename: str or None = get_printable_filename_if_existent(message=message)
        self.content_disposition: str or None = message.get_content_disposition()

        self.child_items = list()
        if message.is_multipart():
            for child in message.get_payload():
                self.child_items.append(StructureItem(message=child))



import argparse
import io
import os
import sys
from cli_formatter.output_formatting import warning, error, info, print_headline_banner

from eml_analyzer.library.outputs import AbstractOutput, StandardOutput, JsonOutput
from eml_analyzer.library.parser import ParsedEmail, EmlParsingException, Attachment


def main():
    argument_parser = argparse.ArgumentParser(prog='emlAnalyzer', description='A CLI script to analyze an email in the EML format for viewing headers, extracting attachments, etc.')
    argument_parser.add_argument('-i', '--input', help="Path to the EML file. Accepts standard input if omitted", type=argparse.FileType('r', encoding='utf-8'), nargs='?', default=sys.stdin)
    argument_parser.add_argument('--header', action='store_true', default=False, help="Shows the headers")
    argument_parser.add_argument('-x', '--tracking', action='store_true', default=False, help="Shows content which is reloaded from external resources in the HTML part")
    argument_parser.add_argument('-a', '--attachments', action='store_true', default=False, help="Lists attachments")
    argument_parser.add_argument('--text', action='store_true', default=False, help="Shows plaintext")
    argument_parser.add_argument('--html', action='store_true', default=False, help="Shows HTML")
    argument_parser.add_argument('-s', '--structure', action='store_true', default=False, help="Shows structure of the E-Mail")
    argument_parser.add_argument('-u', '--url', action='store_true', default=False, help="Shows embedded clickable links and urls in the HTML and text part")
    argument_parser.add_argument('-ea', '--extract', type=int, default=None, help="Extracts the x-th attachment. Can not be used together with the '--format' parameter.")
    argument_parser.add_argument('--extract-all', action='store_true', default=None, help="Extracts all attachments. If a output format is specified the content of the attachments will be included in the structural output as a base64 encoded blob")
    argument_parser.add_argument('-o', '--output', type=str, default=None, help="Path for the extracted attachment (default is filename in working directory)")
    argument_parser.add_argument('--format', default='', const='', nargs='?', choices=['json'], help='Specifies a structured output format, the default format is not machine-readable')
    arguments = argument_parser.parse_args()

    if not arguments.input:
        warning('No input specified')
        argument_parser.print_help()
        exit()

    output_format: AbstractOutput = _get_output_from_cli_arguments_or_exit_on_error(specified_format=arguments.format)

    eml_file = _read_eml_file_or_exit_on_error(output_format=output_format, input_file=arguments.input)
    parsed_email: ParsedEmail = _parse_eml_file_or_exit_on_error(output_format=output_format, eml_content=eml_file)

    # use default functionality if no options are specified
    is_default_functionality = not (arguments.header or
                                    arguments.tracking or
                                    arguments.attachments or
                                    arguments.text or
                                    arguments.html or
                                    arguments.structure or
                                    arguments.url or
                                    arguments.extract is not None)

    if is_default_functionality:
        arguments.structure = True
        arguments.url = True
        arguments.tracking = True
        arguments.attachments = True

    if arguments.header:
        output_format.process_option_show_header(parsed_email=parsed_email)
    if arguments.structure:
        output_format.process_option_show_structure(parsed_email=parsed_email)
    if arguments.url:
        output_format.process_option_show_embedded_urls_in_html_and_text(parsed_email=parsed_email)
    if arguments.tracking:
        output_format.process_option_show_reloaded_content_from_html(parsed_email=parsed_email)
    if arguments.attachments:
        output_format.process_option_show_attachments(parsed_email=parsed_email, extract_content=arguments.extract)
    if arguments.text:
        output_format.process_option_show_text(parsed_email=parsed_email)
    if arguments.html:
        output_format.process_option_show_html(parsed_email=parsed_email)

    if arguments.extract is not None:
        if isinstance(output_format, StandardOutput):
            _extract_attachment(parsed_email=parsed_email, attachment_number=arguments.extract, output_path=arguments.output)
        else:
            output_format.output_error_and_exit(exception=None, error_message="The '--extract' argument can only be used if no output format is specified")

    if arguments.extract_all is not None and isinstance(output_format, StandardOutput):
        _extract_all_attachments(parsed_email=parsed_email, path=arguments.output)

    final_output = output_format.get_final_output(parsed_email=parsed_email)
    if final_output:
        print(final_output)


def _get_output_from_cli_arguments_or_exit_on_error(specified_format: str) -> AbstractOutput:
    if specified_format == '':
        return StandardOutput()
    elif specified_format == 'json':
        return JsonOutput()
    else:
        error('output format is not valid')
        exit()


def _read_eml_file_or_exit_on_error(output_format: AbstractOutput, input_file: io.TextIOWrapper) -> str:
    try:
        with input_file:
            return input_file.read()
    except Exception as e:
        output_format.output_error_and_exit(exception=e, error_message='File could not be loaded')


def _parse_eml_file_or_exit_on_error(output_format: AbstractOutput, eml_content: str) -> ParsedEmail:
    try:
        return ParsedEmail(eml_content=eml_content)
    except EmlParsingException as e:
        output_format.output_error_and_exit(exception=e, error_message='File could not be parsed. Sure it is an eml file?')


def _extract_attachment(parsed_email: ParsedEmail, attachment_number: int, output_path: str or None):
    print_headline_banner('Attachment Extracting')

    attachment = _get_attachment_by_index(parsed_email=parsed_email, attachment_number=attachment_number)

    # Check if attachment was found
    if attachment is None:
        error('Attachment {} could not be found'.format(attachment_number))
        return

    info('Found attachment [{}] "{}"'.format(attachment_number, attachment.filename))

    _write_attachment_to_file(attachment=attachment, output_path=output_path)


def _get_attachment_by_index(parsed_email: ParsedEmail, attachment_number: int) -> Attachment or None:
    attachments = parsed_email.get_attachments()
    for attachment in attachments:
        if attachment.index == attachment_number:
            return attachment
    return None


def _write_attachment_to_file(attachment: Attachment, output_path: str or None) -> None:
    output_path = _get_output_path_for_attachment(attachment=attachment, output_path=output_path)

    output_file = open(output_path, mode='wb')
    output_file.write(attachment.content)
    info('Attachment [{}] "{}" extracted to {}'.format(attachment.index, attachment.filename, output_path))


def _get_output_path_for_attachment(attachment: Attachment, output_path: str or None) -> str:
    if output_path is None:
        return attachment.filename
    elif os.path.isdir(output_path):
        return os.path.join(output_path, attachment.filename)


def _extract_all_attachments(parsed_email: ParsedEmail, path: str or None):
    print_headline_banner('Extracting All Attachments')

    # if no output directory is given then a default directory with the name 'eml_attachments' is used
    if path is None:
        path = 'eml_attachments'

    if not os.path.exists(path):
        os.makedirs(path)

    for attachment in parsed_email.get_attachments():
        _write_attachment_to_file(attachment=attachment, output_path=path)


if __name__ == '__main__':
    main()



import abc

from eml_analyzer.library.parser.parsed_email import ParsedEmail


class AbstractOutput(abc.ABC):
    @abc.abstractmethod
    def get_final_output(self, parsed_email: ParsedEmail) -> str or None:
        pass

    @abc.abstractmethod
    def process_option_show_header(self, parsed_email: ParsedEmail) -> None:
        pass

    @abc.abstractmethod
    def process_option_show_structure(self, parsed_email: ParsedEmail) -> None:
        pass

    @abc.abstractmethod
    def process_option_show_embedded_urls_in_html_and_text(self, parsed_email: ParsedEmail) -> None:
        pass

    @abc.abstractmethod
    def process_option_show_html(self, parsed_email: ParsedEmail) -> None:
        pass

    @abc.abstractmethod
    def process_option_show_text(self, parsed_email: ParsedEmail) -> None:
        pass

    @abc.abstractmethod
    def process_option_show_attachments(self, parsed_email: ParsedEmail, extract_content: bool = False) -> None:
        pass

    @abc.abstractmethod
    def process_option_show_reloaded_content_from_html(self, parsed_email: ParsedEmail) -> None:
        pass

    def output_error_and_exit(self, exception: Exception or None, error_message: str) -> None:
        self.output_error(exception=exception, error_message=error_message)
        exit()

    @abc.abstractmethod
    def output_error(self, exception: Exception or None, error_message: str) -> None:
        pass



import setuptools

with open('README.md', mode='r', encoding='utf-8') as readme_file:
    long_description = readme_file.read()
    # remove badges in Readme
    long_description = '# emlAnalyzer' + long_description.split('# emlAnalyzer')[1]


setuptools.setup(
    name="eml-analyzer",
    version="3.0.1",
    author="Florian Wahl",
    author_email="florian.wahl.developer@gmail.com",
    description="A cli script to analyze an E-Mail in the eml format for viewing the header, extracting attachments, etc.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/wahlflo/eml_analyzer",
    packages=setuptools.find_packages(exclude=("tests", "tests.*")),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
       'cli-formatter>=1.2.0'
    ],
    entry_points={
        "console_scripts": [
            "emlAnalyzer=eml_analyzer.cli_script:main"
        ],
    }
)